from abc import ABCMeta, abstractmethod

from sklearn.base import BaseEstimator


class AbstractEstimator(BaseEstimator, metaclass=ABCMeta):
    @abstractmethod
    def fit(*args, **kwargs):
        pass

    @abstractmethod
    def predict(*args, **kwargs):
        pass

    def predict_proba(*args, **kwargs):
        pass



import functools
from functools import partial


def trackcalls(func):
    """Helper function to check if
    a method already called or not
    Parameters
    ----------
    func : A function or method
    """

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        wrapper.has_been_called = True
        return func(*args, **kwargs)

    wrapper.has_been_called = False
    return wrapper



from abc import ABCMeta, abstractmethod


class OptimizerFactory(metaclass=ABCMeta):
    @abstractmethod
    def optimizer_builder(*args, **kargs):
        """
        Return a optimizer CV instanse, ready to use
        ...

        Attributes
        ----------
        *args: list
            A list of possible argumnets
        **kwargs: dict
            A dict of possible argumnets
        """
        pass



from lohrasb import logger


def _trail_param_retrive(trial, dict, keyword):
    """An internal function. Return a trial suggest using dict params of estimator and
    one keyword of it. Based on the keyword, it will return an
    Optuna.trial.suggest. The return will be trial.suggest_int(keyword, min(dict[keyword]), max(dict[keyword]))

    Example : _trail_param_retrive(trial, {
            "max_depth": [2, 3],
            "min_child_weight": [0.1, 0.9],
            "gamma": [1, 9],
             }, "gamma") --> will be trail.suggest_int for gamma using [1,9]

    Parameters
    ----------
    trial: Optuna trial
        A trial is a process of evaluating an objective function.
        For more info, visit
        https://optuna.readthedocs.io/en/stable/reference/generated/optuna.trial.Trial.html
    dict: dict
        A dictionary of estimator params.
        e.g., {
            "max_depth": [2, 3],
            "min_child_weight": [0.1, 0.9],
            "gamma": [1, 9],
             }
    Keyword: str
        A keyword of estimator key params. e.g., "gamma"
    """
    if isinstance(dict[keyword][0], str) or dict[keyword][0] is None:
        return trial.suggest_categorical(keyword, dict[keyword])
    if isinstance(dict[keyword][0], int):
        if len(dict[keyword]) >= 2:
            if isinstance(dict[keyword][1], int):
                return trial.suggest_int(
                    keyword, min(dict[keyword]), max(dict[keyword])
                )
        else:
            return trial.suggest_float(keyword, min(dict[keyword]), max(dict[keyword]))
    if isinstance(dict[keyword][0], float):
        return trial.suggest_float(keyword, min(dict[keyword]), max(dict[keyword]))


def _trail_params_retrive(trial, dict):
    """An internal function. Return a trial suggests using dict params of estimator.

    Example : _trail_param_retrive(trial, {
            "eval_metric": ["auc"],
            "max_depth": [2, 3],
            "min_child_weight": [0.1, 0.9],
            "gamma": [1, 9],
            "booster": ["gbtree", "gblinear", "dart"],
             }, "gamma") --> will return params where

             parmas = {
                "eval_metric": trial.suggest_categorical("eval_metric", ["auc"]),
                "max_depth": trial.suggest_int("max_depth", 2,3),
                "min_child_weight": trial.suggest_float("min_child_weight", 0.1, 0.9),
                "booster": trial.suggest_categorical("booster", ["gbtree", "gblinear", "dart"]),
             }
    Parameters
    ----------
    trial: Optuna trial
        A trial is a process of evaluating an objective function.
        For more info, visit
        https://optuna.readthedocs.io/en/stable/reference/generated/optuna.trial.Trial.html
    dict: dict
        A dictionary of estimator params.
        e.g., {
            "eval_metric": ["auc"],
            "max_depth": [2, 3],
            "min_child_weight": [0.1, 0.9],
            "gamma": [1, 9],
            "booster": ["gbtree", "gblinear", "dart"],
             }
    """
    params = {}
    for keyword in dict.keys():
        if keyword not in params.keys():
            # if not isinstance(dict[keyword],list):
            #     params[keyword] = trial.set_user_attr(keyword, dict[keyword])
            try:
                if isinstance(dict[keyword][0], str) or dict[keyword][0] is None:
                    params[keyword] = trial.suggest_categorical(keyword, dict[keyword])
                if isinstance(dict[keyword][0], int):
                    if len(dict[keyword]) >= 2:
                        if isinstance(dict[keyword][1], int):
                            params[keyword] = trial.suggest_int(
                                keyword, min(dict[keyword]), max(dict[keyword])
                            )
                    elif len(dict[keyword]) == 1:
                        if isinstance(dict[keyword][0], int):
                            params[keyword] = trial.suggest_int(
                                keyword, min(dict[keyword]), max(dict[keyword])
                            )
                    else:
                        params[keyword] = trial.suggest_float(
                            keyword, min(dict[keyword]), max(dict[keyword])
                        )
                if isinstance(dict[keyword][0], float):
                    params[keyword] = trial.suggest_float(
                        keyword, min(dict[keyword]), max(dict[keyword])
                    )
            except Exception as e:
                logger.error(
                    "the parameters is not a list! it will treated as a scaler"
                )

    return params



import inspect
import os

import numpy as np
from sklearn.metrics import *

from lohrasb import logger

# for load Environment Variables
# True there will not be default args for metric


def f1_plus_tp(y_true, y_pred):
    """Return f1_score+True Possitive

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """
    f1 = f1_score(y_true, y_pred)
    _, _, _, tp = confusion_matrix(y_true, y_pred, labels=[0, 1]).ravel()
    return f1 + tp


def f1_plus_tn(y_true, y_pred):
    """Return f1_score+True Negative

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """

    f1 = f1_score(y_true, y_pred)
    tn, _, _, _ = confusion_matrix(y_true, y_pred, labels=[0, 1]).ravel()
    return f1 + tn


def specificity(y_true, y_pred):
    """Return Specificity

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """

    tn, fp, fn, tp = confusion_matrix(y_true, y_pred, labels=[0, 1]).ravel()
    return tn / (tn + fp)


def tn_score(y_true, y_pred):
    """Return Specificity

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """

    tn, fp, fn, tp = confusion_matrix(y_true, y_pred, labels=[0, 1]).ravel()
    return tn


def tn(y_true, y_pred):
    """Return Specificity

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """

    tn, fp, fn, tp = confusion_matrix(y_true, y_pred, labels=[0, 1]).ravel()
    return tn


def tp_score(y_true, y_pred):
    """Return Specificity

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """

    tn, fp, fn, tp = confusion_matrix(y_true, y_pred, labels=[0, 1]).ravel()
    return tp


def tp(y_true, y_pred):
    """Return Specificity

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """

    tn, fp, fn, tp = confusion_matrix(y_true, y_pred, labels=[0, 1]).ravel()
    return tp


def roc_plus_f1(y_true, y_pred):
    """Return ROC + f1_score

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """
    f1 = f1_score(y_true, y_pred)
    roc = roc_curve(y_true, y_pred)
    return np.sum(f1 + roc)


def auc_plus_f1(y_true, y_pred):
    """Return AUC + f1_score

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """
    f1 = f1_score(y_true, y_pred)
    fpr, tpr, thresholds = roc_curve(y_true, y_pred)
    auc_result = auc(fpr, tpr)
    return f1 + auc_result


def det_curve_ret(y_true, y_pred):
    """Return det_curve

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """
    fpr, fnr, thresholds = det_curve(y_true, y_pred)
    return np.sum(fpr) + np.sum(fnr)


def precision_recall_curve_ret(y_true, y_pred):
    """Return precision_recall_curve

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """
    precision, recall, thresholds = precision_recall_curve(y_true, y_pred)
    return np.sum(precision) + np.sum(recall)


def precision_recall_fscore_support_ret(y_true, y_pred):
    """Return precision_recall_fscore_support

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """
    output = precision_recall_fscore_support(y_true, y_pred)
    return np.sum(output)


def roc_curve_ret(y_true, y_pred):
    """Return roc_curve

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    """
    fpr, tpr, thresholds = roc_curve(y_true, y_pred)
    return np.sum(tpr) - np.sum(fpr)


class CalcMetrics:
    """Return roc_curve

    Parameters
    ----------
    y_true : Pandas DataFrame or Pandas Series
        True values
    y_pred : Pandas DataFrame Pandas Series
        predicted values.
    metric : Str
        Name of a metric function, e.g., f1_score.
    d : dict
        A mapping dictionary that maps a metric function's string name
         to its representative function, e.g., "accuracy_score": accuracy_score.
    """

    def __init__(
        self,
        y_true,
        y_pred,
        metric,
        d={
            "accuracy_score": accuracy_score,  # normal
            "auc": auc,  # normal
            "precision_recall_curve": precision_recall_curve,  # normal
            "balanced_accuracy_score": balanced_accuracy_score,  # normal
            "cohen_kappa_score": cohen_kappa_score,  # normal
            "dcg_score": dcg_score,  # normal
            "det_curve": det_curve_ret,  # normal minimize
            "f1_score": f1_score,  # normal
            "fbeta_score": fbeta_score,  # normal
            "hamming_loss": hamming_loss,  # normal minimize
            "fbeta_score": fbeta_score,  # normal
            "jaccard_score": jaccard_score,  # normal
            "matthews_corrcoef": matthews_corrcoef,  # normal
            "ndcg_score": ndcg_score,  # normal
            "precision_score": precision_score,  # normal
            "recall_score": recall_score,  # normal
            "recall": recall_score,  # normal
            "roc_auc_score": roc_auc_score,  # normal
            "roc_curve": roc_curve_ret,  # normal
            "top_k_accuracy_score": top_k_accuracy_score,  # normal
            "zero_one_loss": zero_one_loss,  # normal minimize
            # customs
            "tn": tn,  # custom
            "tp": tp,  # custom
            "tn_score": tn_score,  # custom
            "tp_score": tp_score,  # custom
            "f1_plus_tp": f1_plus_tp,  # custom
            "f1_plus_tn": f1_plus_tn,  # custom
            "specificity": specificity,  # custom
            "roc_plus_f1": roc_plus_f1,  # custom
            "auc_plus_f1": auc_plus_f1,  # custom
            "precision_recall_curve_ret": precision_recall_curve_ret,  # custom
            "precision_recall_fscore_support": precision_recall_fscore_support_ret,  # custom
            # regression
            "explained_variance_score": explained_variance_score,
            "max_error": max_error,
            "mean_absolute_error": mean_absolute_error,
            "mean_squared_log_error": mean_squared_log_error,
            "mean_absolute_percentage_error": mean_absolute_percentage_error,
            "mean_squared_log_error": mean_squared_log_error,
            "median_absolute_error": median_absolute_error,
            "mean_absolute_percentage_error": mean_absolute_percentage_error,
            "r2_score": r2_score,
            "mean_poisson_deviance": mean_poisson_deviance,
            "mean_gamma_deviance": mean_gamma_deviance,
            "mean_tweedie_deviance": mean_tweedie_deviance,
            "d2_tweedie_score": d2_tweedie_score,
            "mean_pinball_loss": mean_pinball_loss,
            "d2_pinball_score": d2_pinball_score,
            "d2_absolute_error_score": d2_absolute_error_score,
        },
        change_default_args_of_metric=False,
        *args,
        **kwargs,
    ):
        self.y_true = y_true
        self.y_pred = y_pred
        self.metric = metric
        self.d = d
        # set change_default_args_of_metric based on debug mode
        self.change_default_args_of_metric = change_default_args_of_metric
        self.args = args
        self.kwargs = kwargs

    def resolve_name(self):
        """
        Return a metric function.
        """
        if self.d[self.metric]:
            return self.d[self.metric]

    def get_func_args(self):
        """
        Return a dictionary of arguments of a metric function.
        """
        args_of_func = inspect.signature(self.resolve_name()).parameters
        return args_of_func

    def get_func_default_args(self):
        """
        Return a dictionary of all default arguments of a metric function.
        """
        args_of_func = self.get_func_args()
        d = {}
        for name, value in args_of_func.items():
            value_str = str(value)
            if (
                "=" in value_str
                and str(value) != "y_true"
                and str(value) != "y_score"
                and str(value) != "y_pred"
                and str(value) != "y_prob"
            ):
                d[name] = value
        func_default_args = d
        return func_default_args

    def get_transformed_default_args(self):
        """
        Return a fixed dictionary of all default arguments of a metric function.
        """
        func_default_args = self.get_func_default_args()
        d = {}
        for name, value in func_default_args.items():
            value_str = str(value)
            if "None" in value_str:
                d[name] = None
            elif "True" in value_str:
                d[name] = True
            elif "False" in value_str:
                d[name] = False
            elif "'" in value_str and "=" in value_str:
                start = end = "'"
                d[name] = value_str.split(start)[1].split(end)[0]
            elif "'" not in value_str and "=" in value_str:
                start = end = "="
                d[name] = value_str.split(start)[1].split(end)[0]
            for k, v in func_default_args.items():
                try:
                    start = end = "="
                    str_var = str(v).split(start)[1].split(end)[0]
                    float_str = float(str_var)
                    d[k] = float_str
                except ValueError:
                    logger.warning(f"Warning! {k} is not used in {self.metric}")
                try:
                    start = end = "="
                    str_var = str(v).split(start)[1].split(end)[0]
                    int_str = int(str_var)
                    d[k] = int_str
                except ValueError:
                    logger.warning(f"Warning! {k} is not used in {self.metric}")

        transformed_defualt_args = d
        return transformed_defualt_args

    def asign_default(self):
        """
        Provide a way for users to change default values of default
        arguments of a metric function.
        """
        metric = self.resolve_name()
        transformed_defualt_args = self.get_transformed_default_args()
        if self.change_default_args_of_metric:
            if len(transformed_defualt_args) > 0:
                ans = input(
                    f"Do you want to change defualt arguments for {str(metric.__name__)} Y/N? : "
                )
                if ans == "Y" or ans == "y" or ans == "yes":
                    self.change_default_args_of_metric = True
                else:
                    self.change_default_args_of_metric = False
                if self.change_default_args_of_metric:
                    for t, v in transformed_defualt_args.items():
                        value_input = input(
                            f"Set value for {t} of {str(metric.__name__)} : "
                        )
                        if value_input == "":
                            transformed_defualt_args[t] = v
                        elif "True" == value_input or "true" == value_input:
                            transformed_defualt_args[t] = True
                        elif "False" == value_input or "false" == value_input:
                            transformed_defualt_args[t] = False
                        elif "None" == value_input or "none" == value_input:
                            transformed_defualt_args[t] = None
                        else:
                            transformed_defualt_args[t] = str(value_input)

                    for t, v in transformed_defualt_args.items():
                        if not isinstance(v, bool):
                            try:
                                float_v = float(v)
                                transformed_defualt_args[t] = float_v
                            except Exception as e:
                                logger.warning(
                                    f"Warning! {t} is not used in {self.metric}"
                                )
                            try:
                                int_v = int(v)
                                transformed_defualt_args[t] = int_v
                            except Exception as e:
                                logger.warning(
                                    f"Warning! {t} is not used in {self.metric}"
                                )

        return transformed_defualt_args

    def get_default_params_if_any(self):
        """
        Return a default params of  metric function.
        """
        asign_default = self.asign_default()
        return asign_default

    def get_func_string_if_any(self):
        """
        Return a metrics function if any.
        """
        metric = self.resolve_name()
        func = str(metric.__name__)
        return func

    def get_metric_func(self):
        """
        Return a metric function and its default assigned arguments.
        """
        asign_default = self.asign_default()
        metric = self.resolve_name()
        func = str(metric.__name__)
        f_str = func + "(" + "self.y_true,self.y_pred, **asign_default" + ")"
        return eval(f_str)

    def calc_make_scorer(self, metric):
        """
        Calculate function body of a function using make_scorer
        see:
            https://scikit-learn.org/stable/modules/generated/sklearn.metrics.make_scorer.html

        Parameters:
        -----------
        metric: str
            String representation of a metric function.
        """
        metrics_list_for_maximizing = [
            "accuracy_score",
            "auc",
            "precision_recall_curve",
            "balanced_accuracy_score",
            "cohen_kappa_score",
            "dcg_score",
            "f1_score",
            "fbeta_score",
            "fbeta_score",
            "jaccard_score",
            "matthews_corrcoef",
            "ndcg_score",
            "precision_score",
            "recall_score",
            "roc_auc_score",
            "roc_curve",
            "top_k_accuracy_score",
            # customs
            "tn",
            "tn_score",
            "tp",
            "tp_score",
            "f1_plus_tp",
            "f1_plus_tn",
            "specificity",
            "roc_plus_f1",
            "auc_plus_f1",
            "precision_recall_curve",
            "precision_recall_fscore_support",
            "max_error",
            "r2_score",
        ]
        metrics_list_for_minimizing = [
            "det_curve",
            "hamming_loss",
            "zero_one_loss",
            "explained_variance_score",
            "mean_absolute_error",
            "mean_squared_log_error",
            "mean_absolute_percentage_error",
            "mean_squared_log_error",
            "median_absolute_error",
            "mean_absolute_percentage_error",
            "mean_poisson_deviance",
            "mean_gamma_deviance",
            "mean_tweedie_deviance",
            "d2_tweedie_score",
            "mean_pinball_loss",
            "d2_pinball_score",
            "d2_absolute_error_score",
        ]

        if (
            metric.__class__.__name__ == "_BaseScorer"
            or metric.__class__.__name__ == "_ProbaScorer"
            or metric.__class__.__name__ == "_PredictScorer"
            or metric.__class__.__name__ == "_ThresholdScorer"
        ):
            return metric
        elif isinstance(metric, str):
            if "(" in metric:
                metric = metric.replace("y_true", "self.y")
                return metric
            else:
                raise TypeError(
                    f"selected {metric} is not proper. Read examples and documentations !"
                )
        else:
            raise TypeError(
                f"selected {metric} is not proper. Read examples and documentations !"
            )

    def get_simple_metric(self, metric, y_true, y_pred, params=None):
        """
        Return a simple metric of y_true, y_pred
        Parameters:
        -----------
        metric: str
            String representation of a metric function.
        y_true : Pandas DataFrame or Pandas Series
            True values
        y_pred : Pandas DataFrame Pandas Series
            predicted values.
        """
        if params is None:
            f_str = metric + "(" + "y_true,y_pred" + ")"
            return eval(f_str)
        if isinstance(params, dict) and len(params) == 0:
            f_str = metric + "(" + "y_true,y_pred" + ")"
            return eval(f_str)
        if isinstance(params, dict) and len(params) > 0:
            f_str = metric + "(" + "y_true,y_pred,**params" + ")"
            return eval(f_str)
        return None


if __name__ == "__main__":
    # test metrics
    calcmetric = CalcMetrics(
        y_true=np.array([0, 0, 1, 0]),
        y_pred=np.array([0, 1, 1, 0]),
        metric="d2_absolute_error_score",
    )
    calcmetric.get_metric_func()
    func = calcmetric.calc_make_scorer("d2_absolute_error_score")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lohrasb',
 'lohrasb.abstracts',
 'lohrasb.base_classes',
 'lohrasb.decorators',
 'lohrasb.examples',
 'lohrasb.factories',
 'lohrasb.mediators',
 'lohrasb.utils']

package_data = \
{'': ['*']}

install_requires = \
['category-encoders>=2.5.0,<3.0.0',
 'feature-engine>=1.4.1,<2.0.0',
 'hyperopt>=0.2.7,<0.3.0',
 'imblearn>=0.0,<0.1',
 'importlib>=1.0.4,<2.0.0',
 'interpret>=0.3.0,<0.4.0',
 'lightgbm>=3.3.2,<4.0.0',
 'nox>=2022.1.7,<2023.0.0',
 'numpy>=1.20,<2.0',
 'optuna>=2.10.1,<3.0.0',
 'pandas>=1.4,<2.0',
 'protobuf==3.20',
 'pyarrow>=10.0.1,<11.0.0',
 'pycox>=0.2.3,<0.3.0',
 'python-dotenv>=0.21.1,<0.22.0',
 'ray[tune]>=2.2.0,<3.0.0',
 'scikit-learn>=1.1.3,<2.0.0',
 'scikit-optimize>=0.9.0,<0.10.0',
 'scipy>=1.5,<2.0',
 'tune-sklearn>=0.4.5,<0.5.0',
 'xgboost>=1.6.1,<2.0.0',
 'xgbse>=0.2.3,<0.3.0']

setup_kwargs = {
    'name': 'lohrasb',
    'version': '3.4.0',
    'description': 'A scalable estimator optimization tool compatible with scikit-learn APIs',
    'long_description': '![GitHub Repo stars](https://img.shields.io/github/stars/drhosseinjavedani/lohrasb) ![GitHub forks](https://img.shields.io/github/forks/drhosseinjavedani/lohrasb) ![GitHub language count](https://img.shields.io/github/languages/count/drhosseinjavedani/lohrasb) ![GitHub repo size](https://img.shields.io/github/repo-size/drhosseinjavedani/lohrasb) ![GitHub](https://img.shields.io/github/license/drhosseinjavedani/lohrasb)![PyPI - Downloads](https://img.shields.io/pypi/dd/lohrasb) ![PyPI - Python Version](https://img.shields.io/pypi/pyversions/lohrasb) \n\n# Lohrasb\n\nLohrasb is a  tool built to ease machine learning development by tuning hyper-parameters of estimators in a scalable way. It uses [Optuna](https://optuna.readthedocs.io/en/stable/index.html), [GridSearchCV](https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.GridSearchCV.html), [RandomizedSearchCV](https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html), and [Ray tune Scikit-Learn API](https://docs.ray.io/en/latest/tune/api_docs/sklearn.html) to tune most of the estimators of sickit-learn. It is compatible with [scikit-learn](https://scikit-learn.org) pipeline, [XGBoost Survival Embeddings](https://github.com/loft-br/xgboost-survival-embeddings) and, [InterpretML](https://github.com/interpretml/interpret/).\n\n\n### Introduction\n\nBaseModel of the Lohrasb package can receive various parameters. From an estimator class to its tunning parameters and GridsearchCV, RandomizedSearchCV, or Optuna to their parameters. Samples will be split to train and validation set, and then optimization will estimate optimal related parameters using these optimizing engines.\n\n### Installation\n\nLohrasb package is available on PyPI and can be installed with pip:\n\n```sh\npip install lohrasb\n```\n\n\n### Supported estimators for this package\nLohrasb supports almost all machine learning estimators for classification and regression.\n\n### Usage\n\n- Tunning best parameters of a machine learning model using [Optuna](https://optuna.readthedocs.io/en/stable/index.html) , [GridSearchCV](https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.GridSearchCV.html) [RandomizedSearchCV](https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.RandomizedSearchCV.html), [TuneGridSearchCV, and TuneSearchCV](https://docs.ray.io/en/latest/tune/api_docs/sklearn.html) from [Ray](https://docs.ray.io/en/latest/index.html) tune Scikit-Learn API (tune.sklearn).\n\n### Factories\nFor ease of use of BestModel, some factories are available to build associated instances corresponding to each optimization engine. For example, the following factory can be used for  GridSearchCV:\n\n```\n\nobj = BaseModel().optimize_by_gridsearchcv(\n            estimator=XGBClassifier(),\n            estimator_params={\n                                "booster": ["gbtree","dart"],\n                                "eval_metric": ["auc"],\n                                "max_depth": [4, 5],\n                                "gamma": [0.1, 1.2],\n                                "subsample": [0.8],\n                            },\n            fit_params = None,\n            measure_of_accuracy=make_scorer(f1_score, greater_is_better=True),\n            # instead of measure_of_accuracy it is possible to use :  scoring="f1",\n            verbose=3,\n            n_jobs=-1,\n            random_state=42,\n            cv=KFold(2),\n        )\n```\n\n#### Example 1: Computer Hardware (Part 1: Use BestModel in sklearn pipeline)\n\n##### Import some required libraries\n```\nfrom lohrasb.best_estimator import BaseModel\nimport xgboost\nfrom optuna.pruners import HyperbandPruner\nfrom optuna.samplers._tpe.sampler import TPESampler\nfrom sklearn.model_selection import KFold,train_test_split\nimport pandas as pd\nimport numpy as np\nimport optuna\nfrom sklearn.pipeline import Pipeline\nfrom feature_engine.imputation import (\n    CategoricalImputer,\n    MeanMedianImputer\n    )\nfrom category_encoders import OrdinalEncoder\nfrom sklearn.metrics import (\n    make_scorer)\nfrom sklearn.metrics import r2_score\nfrom sklearn.linear_model import *\n\n\n```\n##### Computer Hardware [Data Set](https://archive.ics.uci.edu/ml/datasets/Computer+Hardware)\n (a regression problem)\n  \n\n```\nurldata= "https://archive.ics.uci.edu/ml/machine-learning-databases/cpu-performance/machine.data"\n# column names\ncol_names=[\n    "vendor name",\n    "Model Name",\n    "MYCT",\n    "MMIN",\n    "MMAX",\n    "CACH",\n    "CHMIN",\n    "CHMAX",\n    "PRP"\n]\n# read data\ndata = pd.read_csv(urldata,header=None,names=col_names,sep=\',\')\ndata\n```\n##### Train test split\n\n```\nX = data.loc[:, data.columns != "PRP"]\ny = data.loc[:, data.columns == "PRP"]\ny = y.values.ravel()\n\n\nX_train, X_test, y_train, y_test =train_test_split(X, y, test_size=0.33, random_state=42)\n\n```\n##### Find feature types for later use\n\n```\nint_cols =  X_train.select_dtypes(include=[\'int\']).columns.tolist()\nfloat_cols =  X_train.select_dtypes(include=[\'float\']).columns.tolist()\ncat_cols =  X_train.select_dtypes(include=[\'object\']).columns.tolist()\n```\n\n#####  Define estimator and set its arguments  \n```\nestimator = LinearRegression()\nestimator_params= {\n        "fit_intercept": [True, False],\n    }\n```\n##### Use factory\n\n```\nobj = BaseModel().optimize_by_optuna(\n            estimator=estimator,\n            estimator_params=estimator_params,\n            measure_of_accuracy="mean_absolute_error(y_true, y_pred, multioutput=\'uniform_average\')",\n            with_stratified=False,\n            test_size=.3,\n            verbose=3,\n            n_jobs=-1,\n            random_state=42,\n            # optuna params\n            # optuna study init params\n            study=optuna.create_study(\n                storage=None,\n                sampler=TPESampler(),\n                pruner=HyperbandPruner(),\n                study_name=None,\n                direction="minimize",\n                load_if_exists=False,\n                directions=None,\n            ),\n            # optuna optimization params\n            study_optimize_objective=None,\n            study_optimize_objective_n_trials=10,\n            study_optimize_objective_timeout=600,\n            study_optimize_n_jobs=-1,\n            study_optimize_catch=(),\n            study_optimize_callbacks=None,\n            study_optimize_gc_after_trial=False,\n            study_optimize_show_progress_bar=False,\n        )\n```\n#####  Build sklearn pipeline\n```\npipeline =Pipeline([\n            # int missing values imputers\n            (\'intimputer\', MeanMedianImputer(\n                imputation_method=\'median\', variables=int_cols)),\n            # category missing values imputers\n            (\'catimputer\', CategoricalImputer(variables=cat_cols)),\n            #\n            (\'catencoder\', OrdinalEncoder()),\n            # regression model \n            (\'obj\', obj),\n\n\n ])\n\n\n\n```\n##### Run Pipeline\n\n```\npipeline.fit(X_train,y_train)\ny_pred = pipeline.predict(X_test)\n```\n##### Check performance of the pipeline\n\n```\nprint(\'r2 score : \')\nprint(r2_score(y_test,y_pred))\n```\n\n##### Part 2: Use BestModel as a standalone estimator \n```\nX_train, X_test, y_train, y_test =train_test_split(X, y, test_size=0.33, random_state=42)\n```\n\n##### Transform features to make them ready for model input\n```\ntransform_pipeline =Pipeline([\n            # int missing values imputers\n            (\'intimputer\', MeanMedianImputer(\n                imputation_method=\'median\', variables=int_cols)),\n            # category missing values imputers\n            (\'catimputer\', CategoricalImputer(variables=cat_cols)),\n            #\n            (\'catencoder\', OrdinalEncoder()),\n            # classification model\n\n ])\n```\n##### Transform X_train and X_test\n\n```\nX_train=transform_pipeline.fit_transform(X_train,y_train)\nX_test=transform_pipeline.transform(X_test)\n```\n\n##### Train model and predict\n```\nobj.fit(X_train,y_train)\ny_pred = obj.predict(X_test)\n```\n\n##### Check performance of the model\n\n```\nprint(\'r2 score : \')\nprint(r2_score(y_test,y_pred))\nprint(\'mean_absolute_error : \')\nprint(mean_absolute_error(y_test,y_pred))\n\n\nprint(obj.get_best_estimator())\n\nprint(obj.best_estimator)\n\nOptunaObj = obj.get_optimized_object()\nprint(OptunaObj.trials)\n```\n\n#### Example 2: XGBoost Survival Embeddings (XGBSEKaplanNeighbors)\nFor more information refer to visit [XGBoost Survival Embeddings](https://loft-br.github.io/xgboost-survival-embeddings/examples/confidence_interval.html) page.\n\n\n##### Import some required libraries\n```\n! pip3 install torch==1.12.1\nimport sys\nsys.path.append(\'/usr/local/lib/python3.10/site-packages\')\nimport torch\nimport numpy as np\nfrom sklearn.model_selection import KFold, train_test_split\nfrom lohrasb.best_estimator import BaseModel\nfrom sklearn.pipeline import Pipeline\nfrom sklearn.metrics import make_scorer\nfrom xgbse.converters import convert_to_structured\nfrom xgbse.metrics import (\n    concordance_index,\n    approx_brier_score\n)\nfrom xgbse import (\n    XGBSEKaplanNeighbors,\n    XGBSEKaplanTree,\n    XGBSEBootstrapEstimator\n)\nfrom pycox.datasets import metabric\n\n```\n\n\n##### Read data metabric\n\n```\ndf = metabric.read_df()\ndf.head()\n```\n\n##### Define labels and train-test split \n\n```\n# splitting to X, T, E format\nX = df.drop([\'duration\', \'event\'], axis=1)\ny = convert_to_structured(df[\'duration\'], df[\'event\'])\n\n\n# splitting between train, and validation \n(X_train, X_test,\n y_train, y_test) = \\\ntrain_test_split(X, y, test_size=0.2, random_state=42)\n```\n\n\n##### Define estimator and set its arguments\n```\nestimator_params = {\n    \'n_estimators\' :[100,200]\n\n}\n\nPARAMS_TREE = {\n    \'objective\': \'survival:cox\',\n    \'eval_metric\': \'cox-nloglik\',\n    \'tree_method\': \'hist\', \n    \'max_depth\': 100, \n    \'booster\':\'dart\', \n    \'subsample\': 1.0,\n    \'min_child_weight\': 50, \n    \'colsample_bynode\': 1.0\n}\nbase_model = XGBSEKaplanTree(PARAMS_TREE)\n\nTIME_BINS = np.arange(15, 315, 15)\n```\n\n##### Define estimator and fit params\n\n```\nestimator=XGBSEBootstrapEstimator(base_model)\nfit_params = {"time_bins":TIME_BINS}\n```\n##### Define BaseModel estimator using random search CV\n\n```\nobj = BaseModel().optimize_by_randomsearchcv(\n            estimator=estimator,\n            fit_params = fit_params,\n            estimator_params=estimator_params,\n            measure_of_accuracy=make_scorer(approx_brier_score, greater_is_better=False),\n            verbose=3,\n            n_jobs=-1,\n            n_iter=2,\n            random_state=42,\n            cv=KFold(2),\n        )\n```\n\n##### Build sklearn pipeline\n\n```\npipeline =Pipeline([\n            (\'obj\', obj)\n\n ])\n```\n##### Run Pipeline\n```\npipeline.fit(X_train,y_train)\ny_pred = pipeline.predict(X_test)\n```\n\n##### Check performance of the pipeline\n\n```\nprint(f\'C-index: {concordance_index(y_test, y_pred)}\')\nprint(f\'Avg. Brier Score: {approx_brier_score(y_test, y_pred)}\')\n```\n\n#### Example 3: Using InterpretML\n\n[InterpretML](https://interpret.ml/docs/getting-started) is a package that supports training interpretable models (glassbox), as well as explaining existing ML pipelines (blackbox). In this example we will use [Shapley Additive Explanations](https://interpret.ml/docs/shap.html) of this package on [Adult Data Set](https://archive.ics.uci.edu/ml/datasets/adult) (a classification problem).\n\n##### Import some required libraries\n\n```\nimport pandas as pd\nfrom sklearn.model_selection import KFold, train_test_split\nfrom lohrasb.best_estimator import BaseModel\nfrom sklearn.pipeline import Pipeline\nfrom feature_engine.imputation import (\n    CategoricalImputer,\n    MeanMedianImputer\n    )\nfrom category_encoders import OrdinalEncoder\nfrom sklearn.metrics import (\n    classification_report,\n    confusion_matrix,\n    f1_score)\nfrom sklearn.metrics import f1_score, make_scorer\nfrom sklearn.ensemble import RandomForestClassifier\nfrom interpret import show\nfrom interpret.blackbox import ShapKernel\nimport shap\nfrom interpret import set_visualize_provider\nfrom interpret.provider import InlineProvider\nset_visualize_provider(InlineProvider())```\n```\n\n##### Read data\n```\nurldata= "https://archive.ics.uci.edu/ml/machine-learning-databases/adult/adult.data"\n# column names\ncol_names=["age", "workclass", "fnlwgt" , "education" ,"education-num",\n"marital-status","occupation","relationship","race","sex","capital-gain","capital-loss","hours-per-week",\n"native-country","label"\n]\n# read data\ndata = pd.read_csv(urldata,header=None,names=col_names,sep=\',\')\ndata.head()\n```\n\n##### Define labels\n```\ndata.loc[data[\'label\']==\'<=50K\',\'label\']=0\ndata.loc[data[\'label\']==\' <=50K\',\'label\']=0\n\ndata.loc[data[\'label\']==\'>50K\',\'label\']=1\ndata.loc[data[\'label\']==\' >50K\',\'label\']=1\n\ndata[\'label\']=data[\'label\'].astype(int)\n```\n\n##### Train test split\n```\n\nX = data.loc[:, data.columns != "label"]\ny = data.loc[:, data.columns == "label"]\n\nX_train, X_test, y_train, y_test =train_test_split(X, y, \\\n     test_size=0.33, stratify=y[\'label\'], random_state=42)\n\ny_train=y_train.values.ravel()\ny_test=y_test.values.ravel()\n```\n\n\n##### Find feature types for later use\n```\n\nint_cols =  X_train.select_dtypes(include=[\'int\']).columns.tolist()\nfloat_cols =  X_train.select_dtypes(include=[\'float\']).columns.tolist()\ncat_cols =  X_train.select_dtypes(include=[\'object\']).columns.tolist()\n```\n\n\n##### Define estimator and set its arguments \n```\n\nestimator = RandomForestClassifier()\nestimator_params = {\n        "max_depth": [4, 5],\n        "n_estimators":[100,200],\n        "max_features" :["sqrt", "log2"],\n\n\n    }\n    \n\nblackbox_model = BaseModel().optimize_by_gridsearchcv(\n            estimator=estimator,\n            estimator_params=estimator_params,\n            fit_params = None,\n            measure_of_accuracy=make_scorer(f1_score, greater_is_better=True),\n            verbose=3,\n            n_jobs=-1,\n            random_state=42,\n            cv=KFold(2),\n        )\n```\n\n##### Build sklearn pipeline\n\n\n```\n\npipeline =Pipeline([\n            # int missing values imputers\n            (\'intimputer\', MeanMedianImputer(\n                imputation_method=\'median\', variables=int_cols)),\n            # category missing values imputers\n            (\'catimputer\', CategoricalImputer(variables=cat_cols)),\n            #\n            (\'catencoder\', OrdinalEncoder()),\n            # classification model\n            #(\'obj\', RandomForestClassifier())\n\n ])\n```\n\n\n##### Run Pipeline\n```\n\nX_train = pipeline.fit_transform(X_train,y_train)\nX_test = pipeline.transform(X_test)\n\nblackbox_model.fit(X_train, y_train)\ny_pred = blackbox_model.predict(X_test)\n```\n\n\n##### Check performance of the pipeline\n```\n\nprint(\'F1 score : \')\nprint(f1_score(y_test,y_pred))\nprint(\'Classification report : \')\nprint(classification_report(y_test,y_pred))\nprint(\'Confusion matrix : \')\nprint(confusion_matrix(y_test,y_pred))\n```\n\n\n#####  Interpret of the pipeline and its decisions with SHAP. The visualizations provided will be for local explanations.\n\n(train set is big only we use 500 as a sample)\n```\n\nshap_kernel = ShapKernel(predict_fn=blackbox_model.predict_proba, data=shap.sample(X_train,500))\nshap_local = shap_kernel.explain_local(X_test[:20], y_test[:20])\nshow(shap_local)\n\n```\n\n#### Example 4: Explaining [Ray](https://docs.ray.io/en/latest/index.html) [Scikit-Learn API (tune.sklearn)](https://docs.ray.io/en/latest/tune/api_docs/sklearn.html)\n\n##### imports\n```\nimport pandas as pd\nfrom sklearn.model_selection import KFold, train_test_split\nfrom lohrasb.best_estimator import BaseModel\nfrom sklearn.pipeline import Pipeline\nfrom feature_engine.imputation import (\n    CategoricalImputer,\n    MeanMedianImputer\n    )\nfrom category_encoders import OrdinalEncoder\nfrom sklearn.metrics import (\n    classification_report,\n    confusion_matrix,\n    f1_score)\nfrom sklearn.metrics import f1_score, make_scorer\nfrom xgboost import *\nfrom lohrasb import logger\n\nlogger.info("Using Scikit-Learn API (tune-sklearn) for an example of classification")\n```\n##### Example: Use [Adult Data Set](https://archive.ics.uci.edu/ml/datasets/Adult) (a classification problem)\n\n```\nurldata= "https://archive.ics.uci.edu/ml/machine-learning-databases/adult/adult.data"\n# column names\ncol_names=["age", "workclass", "fnlwgt" , "education" ,"education-num",\n"marital-status","occupation","relationship","race","sex","capital-gain","capital-loss","hours-per-week",\n"native-country","label"\n]\n# read data\ndata = pd.read_csv(urldata,header=None,names=col_names,sep=\',\')\ndata.head()\n```\n##### Define labels\n```\ndata.loc[data[\'label\']==\'<=50K\',\'label\']=0\ndata.loc[data[\'label\']==\' <=50K\',\'label\']=0\n\ndata.loc[data[\'label\']==\'>50K\',\'label\']=1\ndata.loc[data[\'label\']==\' >50K\',\'label\']=1\n\ndata[\'label\']=data[\'label\'].astype(int)\n```\n##### Train test split\n```\nX = data.loc[:, data.columns != "label"]\ny = data.loc[:, data.columns == "label"]\n\n\nX_train, X_test, y_train, y_test =train_test_split(X, y, \\\ntest_size=0.33, stratify=y[\'label\'], random_state=42)\n```\n##### Find feature types for later use\n```\nint_cols =  X_train.select_dtypes(include=[\'int\']).columns.tolist()\nfloat_cols =  X_train.select_dtypes(include=[\'float\']).columns.tolist()\ncat_cols =  X_train.select_dtypes(include=[\'object\']).columns.tolist()\n```\n##### Define estimator and set its arguments \n```\nestimator = XGBClassifier()\nestimator_params = {\n        "booster": ["gbtree","dart"],\n        "eval_metric": ["auc"],\n        "max_depth": [4, 5],\n        "gamma": [0.1, 1.2],\n        "subsample": [0.8],\n\n    }\n\n\nobj = BaseModel().optimize_by_tunesearchcv(\n            estimator=estimator,\n            estimator_params=estimator_params,\n            fit_params = None,\n            measure_of_accuracy=make_scorer(f1_score, greater_is_better=True),\n            verbose=3,\n            random_state=44,\n            n_jobs=None,\n            cv=KFold(3),\n            early_stopping=None, \n            n_trials=10,\n            scoring=None, \n            refit=True, \n            error_score=\'raise\', \n            return_train_score=False, \n            local_dir=\'~/ray_results\', \n            name=None, \n            max_iters=1, \n            search_optimization=\'hyperopt\',\n            use_gpu=False, \n            loggers=None, \n            pipeline_auto_early_stop=True, \n            stopper=None, \n            time_budget_s=None, \n            mode=None,\n            search_kwargs=None, \n\n        )\n```\n##### Build sklearn pipeline\n\n\n```\npipeline =Pipeline([\n            # int missing values imputers\n            (\'intimputer\', MeanMedianImputer(\n                imputation_method=\'median\', variables=int_cols)),\n            # category missing values imputers\n            (\'catimputer\', CategoricalImputer(variables=cat_cols)),\n            #\n            (\'catencoder\', OrdinalEncoder()),\n            # classification model\n            (\'obj\', obj)\n\n ])\n```\n##### Run Pipeline\n```\npipeline.fit(X_train,y_train)\ny_pred = pipeline.predict(X_test)\n```\n##### Check performance of the pipeline\n```\nprint(\'F1 score : \')\nprint(f1_score(y_test,y_pred))\nprint(\'Classification report : \')\nprint(classification_report(y_test,y_pred))\nprint(\'Confusion matrix : \')\nprint(confusion_matrix(y_test,y_pred))\n```\n\n\nThere are some more examples  available in the [examples](https://github.com/drhosseinjavedani/lohrasb/tree/main/lohrasb/examples) webpage. \n\n#### License\nLicensed under the [BSD 2-Clause](https://opensource.org/licenses/BSD-2-Clause) License.',
    'author': 'drhosseinjavedani',
    'author_email': 'h.javedani@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/drhosseinjavedani/lohrasb',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
